#!/usr/bin/python

"""
Support for building rlvm objects
"""

import os

def _MakeStaticName(lib_name):
  return 'STATIC_' + lib_name + "_LIBS"

def RlvmLibrary(env, lib_name, *args, **kwargs):
  """
  Psuedo-builder for rlvm's internal libraries to speed up development.
  """
#   if env['BUILD_TYPE'] == "Debug":
#     return env.SharedLibrary(lib_name, args)
#   else:
  return env.StaticLibrary(lib_name, *args, **kwargs)


def RlvmProgram(env, prog_name, *args, **kwargs):
  """Psuedo-builder for rlvm programs to handle special static linking on per
  platform basis.

  RlvmProgram should be invoked per this example:

    env.RlvmProgram("rlvm", [rlvm objects files], use_lib_set = ["SDL", "LUA"],
                    rlvm_libs = ['guichan_platform', 'rlvm'])

  This will build the 'rlvm' binary with component libs libguichan_platform and
  librlvm while including all the static libraries necessary for SDL and for
  LUA.

  Environment Variables Responded to:
  - FULL_STATIC_BUILD: Checks to see if this is set and will attempt to link
    the static

  - STATIC_{string in set use_lib_set}_LIBS: Passing a string in the
    use_lib_set will add the static libraries in that set to

  - BUILD_TYPE: In "Debug" mode, all the values passed into RLVM_LIBS are
    assumed to be

  Returns:
    Output node list from env.Program().
  """
  # Need to be smarter about cloning
  cloned_env = env.Clone()

  # All the objects to link into the program. Note the order by which we add
  # all these
  objects = [ ]

  # Add the actual program files
  objects.extend(args)

  # TODO: Deal with RLVM_LIBS in a shared objecty way
  if kwargs.has_key('rlvm_libs'):
    for lib_name in kwargs['rlvm_libs']:
      objects.append(env['LIBPREFIX'] + lib_name + env['LIBSUFFIX'])

  # Add all static libraries from the various categories
  if kwargs.has_key('use_lib_set'):
    for lib_set_name in kwargs['use_lib_set']:
      lib_set = cloned_env[_MakeStaticName(lib_set_name)]
      if lib_set:
        objects.extend(lib_set)

  # First, we need to see if this is a static build
  if kwargs.has_key("full_static_build") and kwargs['full_static_build'] == True:
    # We must unpack each entry in LIBS and try to locate a static library to
    old_libs = cloned_env['LIBS']
    libpaths = cloned_env['LIBPATH']
    no_static_library_libs = [ ]

    for lib in old_libs:
      found = False

      if isinstance(lib, str):
        for libpath in libpaths:
          absolute_path = libpath + "/" + env['LIBPREFIX'] + lib + env['LIBSUFFIX']
          if os.path.exists(absolute_path):
            objects.append(absolute_path)
            found = True
            break

      if found == False:
        no_static_library_libs.append(lib)

    # Put libraries with no static version back into the path
    cloned_env['LIBS'] = no_static_library_libs

  out_nodes = cloned_env.Program(prog_name, objects)

  return out_nodes

def AddStaticLibraryTo(env, name, type):
  """
  Adds a library to one of the internal STATIC_name_LIBS tables.
  """
  path = env['LIBRARY_DIR'] + "/" + env['LIBPREFIX'] + name + env['LIBSUFFIX']
  env[_MakeStaticName(type)].insert(0, path)


def BuildSubcomponent(env, component_name):
  """
  Builds the subcomponent |name| and puts the resultant libraries in
  build/libraries/|name| and exposes that subcomponent's headers to |env|.
  """
  component_env = env.Clone()
  component_env.Append(
    CPPFLAGS = [
      "-Os"
    ]
  )

  component_env.SConscript("vendor/" + component_name + "/SConscript",
                           variant_dir="build/libraries/" + component_name,
                           duplicate=0,
                           exports=["component_env", "env"])

  # Make sure the main compilation can see the includes to these files
  env.Append(CPPPATH = [ "#vendor/" + component_name + "/include/" ])


def generate(env, **kw):
  env.Append(
    # A list of absolute paths to static sdl libraries to make things that need
    # SDL work.
    STATIC_SDL_LIBS = [ ],

    # A list of absolute paths to static lua objects to make that subsystem
    # work.
    STATIC_LUA_LIBS = [ ],

    # A list of absolute paths to static google testing libraries.
    STATIC_TEST_LIBS = [ ]
  )

  env.AddMethod(RlvmLibrary)
  env.AddMethod(RlvmProgram)
  env.AddMethod(AddStaticLibraryTo)
  env.AddMethod(BuildSubcomponent)

def exists(env):
  return True



#!/usr/bin/python

import sys
import os
import re

ifndef = re.compile('^\#ifndef')
define = re.compile('^\#define')
endif  = re.compile('^\#endif')

name = sys.argv[1]
bak_name = name + ".bak"

new_guard_name = name.replace('/', '_').replace('.', '_').upper()
new_guard_name += "_"

try:
  os.rename(name, bak_name)

  input = file(bak_name, 'r')
  output = file(name, 'w')

  for line in input:
    m = ifndef.search(line)
    if m is not None:
      output.write('#ifndef ' + new_guard_name + '\n')
      continue

    m = define.search(line)
    if m is not None:
      output.write('#define ' + new_guard_name + '\n')
      continue

    m = endif.search(line)
    if m is not None:
      output.write('#endif  // ' + new_guard_name + '\n')
      continue

    output.write(line)

  input.close()
  output.close()
except IOError:
  sys.stderr.write('Cannot open "%s"\n' % arg)



#!/usr/bin/python
#
# A fairly naive script which dumps out the data of a HIK file to a JSON file
# for editing and reencoding with buildhik.py.

import json
import struct
import sys

def read_string(f):
  length = struct.unpack("i", f.read(0x4))[0]
  format_str = str(length) + "s"
  return struct.unpack(format_str, f.read(length))

property_formats = {
  10100: lambda f: struct.unpack("i", f.read(0x4)),
  10101: lambda f: struct.unpack("i", f.read(0x4)),
  10102: lambda f: struct.unpack("i", f.read(0x4)),
  10103: lambda f: struct.unpack("ii", f.read(0x8)),
  20000: lambda f: struct.unpack("i", f.read(0x4)),
  20001: lambda f: struct.unpack("i", f.read(0x4)),
  20100: read_string,
  20101: lambda f: struct.unpack("ii", f.read(0x8)),
  21000: lambda f: struct.unpack("i", f.read(0x4)),
  21001: lambda f: struct.unpack("iiii", f.read(0x10)),
  21002: lambda f: struct.unpack("iiiii", f.read(0x14)),
  21003: lambda f: struct.unpack("i", f.read(0x4)),
  21100: lambda f: struct.unpack("i", f.read(0x4)),
  21101: lambda f: struct.unpack("iiii", f.read(0x10)),
  21200: lambda f: struct.unpack("i", f.read(0x4)),
  21201: lambda f: struct.unpack("iiii", f.read(0x10)),
  21202: lambda f: struct.unpack("ii", f.read(0x8)),
  21203: lambda f: struct.unpack("i", f.read(0x4)),
  21301: lambda f: struct.unpack("i", f.read(0x4)),
  21300: lambda f: struct.unpack("iiii", f.read(0x10)),
  30000: lambda f: struct.unpack("i", f.read(0x4)),
  30001: lambda f: struct.unpack("i", f.read(0x4)),
  30100: lambda f: struct.unpack("i", f.read(0x4)),
  30101: lambda f: struct.unpack("i", f.read(0x4)),
  30102: lambda f: struct.unpack("i", f.read(0x4)),
  40000: lambda f: struct.unpack("i", f.read(0x4)),
  40101: lambda f: struct.unpack("iiiiiiiiiiiiiiiiiiiiiiiiiiiiiii", f.read(0x7c)),
  40102: lambda f: struct.unpack("i", f.read(0x4)),
  40103: lambda f: struct.unpack("ii", f.read(0x8)),
  40100: lambda f: [read_string(f), struct.unpack("ii", f.read(0x8))],
}

results = [ ]

if len(sys.argv) != 3:
  print "Usage: " + sys.argv[0] + " <input hik file> <output json>"
  exit(-1)

with open(sys.argv[1], "rb") as f:
  # Read past the file magic
  f.read(0x8)

  while True:
    record = [ ]

    raw_property = f.read(0x4)
    if raw_property == '':
      break

    property_id = struct.unpack("i", raw_property)[0]
    while property_id != -1:
      if property_id in property_formats:
        record.append([property_id, property_formats[property_id](f)])
      else:
        print "huh: " + str(property_id)

      property_id = struct.unpack("i", f.read(0x4))[0]

    # Consume the weird 100.
    struct.unpack("i", f.read(0x4))[0]

    results.append(record)

with open(sys.argv[2], "w") as f:
  print >>f, json.dumps(results, sort_keys=True, indent=4)



#!/usr/bin/python
#
# A fairly naive script which takes a JSON file generated from dumphik.py and
# tries to generate a valid HIK file.

import json
import struct
import sys

HIK_HEADER = "1027000010270000".decode('hex')

def write_property(output, property):
  for x in property:
    if isinstance(x, int):
      output.write(struct.pack("i", x))
    elif isinstance(x, str) or isinstance(x, unicode):
      output.write(struct.pack("i", len(x)))
      output.write(x)
    else:
      print type(x)

if len(sys.argv) != 3:
  print "Usage: " + sys.argv[0] + " <input json file> <output hik>"
  exit(-1)

with open(sys.argv[1], "rb") as input:
  data = json.load(input)
  with open(sys.argv[2], "wb") as output:
    output.write(HIK_HEADER)

    for record in data:
      for entry in record:
        property = int(entry[0])
        output.write(struct.pack("i", property))
        write_property(output, entry[1])

      # Write the terminating -1
      output.write(struct.pack("i", -1))

      # Emit the weird 100. TODO(erg): Only bg26o.hik has 100 as the
      # terminating value here. Investigate more.
      output.write(struct.pack("i", 100))

